"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.WebSocketDevClientServer = void 0;

var _WebSocketServer = require("./WebSocketServer");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * Class for creating a WebSocket server for communication with React Native clients.
 * All client logs - logs from React Native application - are processed here.
 *
 * @category Development server
 */
class WebSocketDevClientServer extends _WebSocketServer.WebSocketServer {
  /**
   * Create new instance of WebSocketDevClientServer and attach it to the given Fastify instance.
   * Any logging information, will be passed through standard `fastify.log` API.
   *
   * @param fastify Fastify instance to attach the WebSocket server to.
   */
  constructor(fastify) {
    super(fastify, '/__client');

    _defineProperty(this, "clients", new Map());

    _defineProperty(this, "nextClientId", 0);
  }
  /**
   * Process client message.
   *
   * @param message Stringified client message.
   */


  processMessage(message) {
    const {
      type,
      ...body
    } = JSON.parse(message);

    switch (type) {
      case 'client-log':
        if (body.level === 'error') {
          this.fastify.log.error({
            issuer: 'Console',
            msg: body.data
          });
        } else if (body.level === 'warn') {
          this.fastify.log.warn({
            issuer: 'Console',
            msg: body.data
          });
        } else {
          this.fastify.log.info({
            issuer: 'Console',
            msg: body.data
          });
        }

        break;

      default:
        this.fastify.log.warn({
          msg: 'Unknown client message',
          message
        });
    }
  }
  /**
   * Process new WebSocket connection from client application.
   *
   * @param socket Incoming client's WebSocket connection.
   */


  onConnection(socket) {
    const clientId = `client#${this.nextClientId++}`;
    this.clients.set(clientId, socket);
    this.fastify.log.info({
      msg: 'React Native client connected',
      clientId
    });
    this.clients.set(clientId, socket);

    const onClose = () => {
      this.fastify.log.info({
        msg: 'React Native client disconnected',
        clientId
      });
      this.clients.delete(clientId);
    };

    socket.addEventListener('error', onClose);
    socket.addEventListener('close', onClose);
    socket.addEventListener('message', event => {
      this.processMessage(event.data.toString());
    });
  }

}

exports.WebSocketDevClientServer = WebSocketDevClientServer;
//# sourceMappingURL=WebSocketDevClientServer.js.map